/*
 * All Rights Reserved. Copyright (C) 2008, Hitachi, Ltd.
 */
package webEmployeeinfo;

import java.io.IOException;
import java.io.PrintWriter;
import javax.ejb.EJB;
import javax.ejb.EJBException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import sun.awt.windows.ThemeReader;
import employeeinfo.Employee;
import employeeinfo.EmployeeInfo;
import employeeinfo.MyOptimisticLockException;

// Employee Information Servlet.
public class EmployeeInfoServlet extends HttpServlet {

	@EJB(beanName = "InfoAccess")
	private EmployeeInfo infoAccess;

	public void doPost(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {

		if (req.getParameter("button").equals("Find")) {
			// Processing executed when the "Find" button is clicked.
			this.pushFind(req, res);
		} else if (req.getParameter("button").equals("Update")) {
			// Processing when the "Update" button is clicked.
			this.pushUpdate(req, res);
		}

	}

	private void pushFind(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {

		try {

			Employee infoEmployee = null;
			String message = "";

			String id = req.getParameter("Id").trim();
			if (!id.equals("")) {

				// Finds the data for the Employee.
				infoEmployee = infoAccess.findEntity(Integer.parseInt(id));

				if (infoEmployee == null) {
					message = "ID doesn't exist.";
				}
			}
			this.generatePage(req, res, infoEmployee, message);

		} catch (NumberFormatException nfe) {

			this.generatePage(req, res, null, "Invalid value.");

			nfe.printStackTrace();
			res.setStatus(500);
		} catch (EJBException ex) {

			this.generatePage(req, res, null, "Other EJB errors.");

			ex.printStackTrace();
			res.setStatus(500);
		}

	}

	private void pushUpdate(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {
		Employee infoEmployee = null;
		String message = "xxxxx";

		try {

			String id = req.getParameter("Id").trim();

			String wait = req.getParameter("Wait").trim();

			if (id.equals("")) {
				this.generatePage(req, res, null, "");
				return;
			}

			// Finds the data for the Employee.
			infoEmployee = infoAccess.findEntity(Integer.parseInt(id));

			if (infoEmployee == null) {
				message = "ID doesn't exist.";
			} else {
				String salary = req.getParameter("Salary");
				salary = salary.trim();

				if (salary.equals("")) {
					salary = "0";
				}

				int checkSalary = Integer.parseInt(salary);
				if (checkSalary < 0) {
					throw new NumberFormatException(
							"Invalid value in Salary text box.");
				}
				System.out.println("infoEmployee.getSalary() first         = "
						+ infoEmployee.getSalary());
				System.out.println("checkSalary                            = "
						+ checkSalary);
				infoEmployee.setSalary(infoEmployee.getSalary() + checkSalary);
				/*
				try {
					Thread.sleep(30000);
				} catch (InterruptedException e) {
					// TODO ꂽ catch ubN
					e.printStackTrace();
				}
				*/
				System.out.println("infoEmployee.getSalary() + checkSalary = "
						+ infoEmployee.getSalary());

				// Updates the data for the Employee in the Database.
				infoAccess.updateEntity(infoEmployee);
				System.out.println("infoEmployee.getSalary() second        = "
						+ infoEmployee.getSalary());
				// message = "Update is OK!! : "+ "salary : " +
				// infoEmployee.getSalary() + " " + "version : " + " " +
				// infoEmployee.getEmployeeVersion() + " ";
				message = "Update is OK!! : " + "salary : "
						+ infoEmployee.getSalary() + " ";
				System.out.println("client message : " + message);
			}

			this.generatePage(req, res, infoEmployee, message);

		} catch (NumberFormatException nfe) {
			res.setStatus(501);

			this.generatePage(req, res, null, "Invalid value.");

			message = "Invalid value. : " + "salary : "
					+ infoEmployee.getSalary() + " ";
			System.out.println("NumberFormatException catch client message : " + message);

			// nfe.printStackTrace();
		} catch (MyOptimisticLockException mole) { // Optimistic Lock Exception
			res.setStatus(503);

			this.generatePage(req, res, null,
					"Other users updated data. Please try again.");

			message = "Other users updated data. Please try again. : "
					+ "salary : " + infoEmployee.getSalary() + " ";
			System.out.println("MyOptimisticLockException catch client message : " + message);

			// Refreshes the Employee entity.
			infoAccess.refreshEntity(infoEmployee);

			// mole.printStackTrace();
		} catch (EJBException ex) {
			res.setStatus(500);

			this.generatePage(req, res, null, "Other EJB errors.");
			message = "Other EJB errors. : " + "salary : "
					+ infoEmployee.getSalary() + " ";
			System.out.println("EJBException catch client message : " + message);

			// ex.printStackTrace();
		}

	}

	// Method used to create an HTML.
	private void generatePage(HttpServletRequest req, HttpServletResponse res,
			Employee info, String message) throws ServletException, IOException {

		String id = "";
		String name = "";
		String departmentName = "";
		String salary = "";

		if (info == null) {
			id = req.getParameter("Id");
			if (req.getParameter("button").equals("Find")) {
				salary = "";
			} else {
				salary = req.getParameter("Salary");
			}
		} else {
			id = String.valueOf(info.getId());
			name = info.getName();
			departmentName = info.getDepartment().getDepartmentName();
			salary = String.valueOf(info.getSalary());
		}

		res.setContentType("text/html");
		PrintWriter out = res.getWriter();
		out.println("<html>");
		out.println("<head>");
		out.println("<title>Employee Information</title>");
		out.println("</head>");
		out.println("<body>");
		out.println("<form method = post action=EmployeeInfoAlias>");
		out.println("<h1><b><center>Employee Information</center></b></h1>");
		out.println("<table border=\"0\" align=center>");
		out.println("<tr>");
		out.println("  <td align=\"right\"><b>ID:</b></td>");
		out.println("  <td><input type=\"text\" name=\"Id\" value=\"" + id
				+ "\"></td>");
		out
				.println("  <td><input type=\"submit\" name=\"button\" value=\"Find\"></td>");
		out.println("</tr>");
		out.println("<tr>");
		out.println("  <td align=\"right\"><b>Name:</b></td>");
		out.println("  <td align=\"left\"><b>" + name + "</b></td>");
		out.println("</tr>");
		out.println("<tr>");
		out.println("  <td align=\"right\"><b>Department:</b></td>");
		out.println("  <td align=\"left\"><b>" + departmentName + "</b></td>");
		out.println("</tr>");
		out.println("<tr>");
		out.println("  <td align=\"right\"><b>Salary:</b></td>");
		out.println("  <td><input type=\"text\" name=\"Salary\" value=\""
				+ salary + "\"></td>");
		out
				.println("  <td><input type=\"submit\" name=\"button\" value=\"Update\"></td>");
		out.println("</tr>");
		out.println("</table>");
		out.println("<p>");
		out.println("<font color=\"red\"><b><center>" + message
				+ "</center></b></font>");
		out.println("</form>");
		out.println("</body>");
		out.println("</html>");

	}
}
